
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: C_LFGList, UIParent, unpack, ZoneAbilityFrame

local BuffFrame = BuffFrame
local Mixin = Mixin

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Blizzard Miscellaneous")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	if not self:IsHooked(C_LFGList, "GetSearchResultInfo") then
		self:RawHook(C_LFGList, "GetSearchResultInfo", "C_LFGList_GetSearchResultInfo", true)
	end

	if not self:IsHooked(ZoneAbilityFrame, "UpdateDisplayedZoneAbilities") then
		self:SecureHook(ZoneAbilityFrame, "UpdateDisplayedZoneAbilities", "ZoneAbilityFrame_UpdateDisplayedZoneAbilities")
	end

	self:RegisterFrames()
	self:UpdateCollapseAndExpandButton()
	self:UpdateDefaultPanelPosition()
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE")
end

function M:RegisterFrames()
	A:RegisterFrame("FloatingBattlePetTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingGarrisonFollowerAbilityTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingGarrisonFollowerTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingGarrisonMissionTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingGarrisonShipyardFollowerTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("FloatingPetBattleAbilityTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("ItemRefTooltip"):SetPoint("TOPLEFT", "SixUI_TooltipAnchor", "TOPLEFT")
	A:RegisterFrame("StatusTrackingBarManager"):Hide()
	A:RegisterFrame("TalkingHeadFrame"):SetScale(0.8)
	A:RegisterFrame("ExtraAbilityContainer"):SetScale(0.8):AddCallback(function (_, frame) frame:SetClampedToScreen(false) end)
end

function M:UpdateCollapseAndExpandButton()
	local alpha = (self.config.showCollapseAndExpandButton or self.isClutterKeyDown) and 1 or 0

	BuffFrame.CollapseAndExpandButton:SetAlpha(alpha)
end

function M:UpdateDefaultPanelPosition()
	UIParent:SetAttribute("TOP_OFFSET", -250)
	UIParent:SetAttribute("LEFT_OFFSET", 20)
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	self.isClutterKeyDown = state
	self:UpdateCollapseAndExpandButton()
end

function M:SIXUI_CONFIG_UPDATE(event, module, config, key, ...)
	if module == M:GetName() then
		if key == "showCollapseAndExpandButton" then
			self:UpdateCollapseAndExpandButton()
		end
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:C_LFGList_GetSearchResultInfo(...)
	local searchResultInfo = self.hooks[C_LFGList].GetSearchResultInfo(...)
	return searchResultInfo and Mixin(searchResultInfo, {playstyle = 0}) or nil
end

-- Style the zone ability frame's spell buttons.
function M:ZoneAbilityFrame_UpdateDisplayedZoneAbilities()
	ZoneAbilityFrame:DisableDrawLayer("OVERLAY")

	for spellButton in ZoneAbilityFrame.SpellButtonContainer:EnumerateActive() do
		B:CreateBorder(spellButton, 2)
		spellButton.Icon:SetTexCoord(unpack(C.ZOOM_TEX_COORD))
		spellButton.NormalTexture:SetTexture(nil)
	end
end
